@@ -1,5 +1,6 @@
 /**
- *  Copyright 2014 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -13,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package brut.androlib.res.data;
 
 import brut.androlib.AndrolibException;
@@ -35,12 +35,13 @@ public class ResResSpec {
         this.mId = id;
         String cleanName;
 
-        try {
-            ResResSpec resResSpec = type.getResSpec(name);
+        ResResSpec resResSpec = type.getResSpecUnsafe(name);
+        if (resResSpec != null) {
             cleanName = name + "_APKTOOL_DUPLICATENAME_" + id.toString();
-        } catch (AndrolibException ex) {
+        } else {
             cleanName = (name.isEmpty() ? ("APKTOOL_DUMMYVAL_" + id.toString()) : name);
         }
+        
         this.mName = cleanName;
         this.mPackage = pkg;
         this.mType = type;
