@@ -1,5 +1,6 @@
 /**
- *  Copyright 2014 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -16,7 +17,7 @@
 package brut.androlib;
 
 import brut.androlib.meta.MetaInfo;
-import brut.androlib.res.util.ExtFile;
+import brut.directory.ExtFile;
 import brut.common.BrutException;
 import brut.directory.FileDirectory;
 import brut.util.OS;
@@ -27,11 +28,13 @@ import java.util.Map;
 import java.util.Set;
 import java.util.logging.Logger;
 
+import brut.util.OSDetection;
 import org.custommonkey.xmlunit.*;
 import org.junit.*;
-import static org.junit.Assert.*;
 
-import org.junit.rules.ExpectedException;
+import static org.junit.Assert.*;
+import static org.junit.Assume.*;
+
 import org.xml.sax.SAXException;
 
 import javax.imageio.ImageIO;
@@ -42,7 +45,7 @@ import javax.imageio.ImageIO;
 public class BuildAndDecodeTest {
 
     @BeforeClass
-    public static void beforeClass() throws Exception, BrutException {
+    public static void beforeClass() throws Exception {
         TestUtils.cleanFrameworkFile();
 
         sTmpDir = new ExtFile(OS.createTempDirectory());
@@ -162,6 +165,17 @@ public class BuildAndDecodeTest {
         compareValuesFiles("values-en/strings.xml");
     }
 
+    @Test
+    public void valuesExtraLongExactLengthTest() throws BrutException {
+        Map<String, String> strs = TestUtils.parseStringsXml(new File(sTestNewDir, "res/values-en/strings.xml"));
+
+        // long_string6 should be exactly 0x8888 chars of "a"
+        // the valuesExtraLongTest() should handle this
+        // but such an edge case, want a specific test
+        String aaaa = strs.get("long_string6");
+        assertEquals(0x8888, aaaa.length());
+    }
+
     @Test
     public void crossTypeTest() throws BrutException {
         compareValuesFiles("values-mcc003/strings.xml");
@@ -194,16 +208,26 @@ public class BuildAndDecodeTest {
         compareXmlFiles("res/layout/issue1130.xml");
     }
 
+    @Test
+    public void xmlUniformAutoTextTest() throws BrutException {
+        compareXmlFiles("res/layout/issue1674.xml");
+    }
+
     @Test(expected = AssertionError.class)
     public void xmlFillParentBecomesMatchTest() throws BrutException {
         compareXmlFiles("res/layout/issue1274.xml");
     }
 
+    @Test
+    public void xmlCustomAttrsNotAndroidTest() throws BrutException {
+        compareXmlFiles("res/layout/issue1157.xml");
+    }
+
     @Test
     public void qualifiersTest() throws BrutException {
         compareValuesFiles("values-mcc004-mnc4-en-rUS-ldrtl-sw100dp-w200dp-h300dp"
-                + "-xlarge-long-round-land-desk-night-xhdpi-finger-keyssoft-12key"
-                + "-navhidden-dpad/strings.xml");
+                + "-xlarge-long-round-highdr-land-desk-night-xhdpi-finger-keyssoft-12key"
+                + "-navhidden-dpad-v26/strings.xml");
     }
 
     @Test
@@ -251,6 +275,11 @@ public class BuildAndDecodeTest {
         compareValuesFiles("values-ast/strings.xml");
     }
 
+    @Test
+    public void androidOStringTest() throws BrutException, IOException {
+        compareValuesFiles("values-ast/strings.xml");
+    }
+
     @Test
     public void twoLetterNotHandledAsBcpTest() throws BrutException, IOException {
         checkFolderExists("res/values-fr");
@@ -287,6 +316,28 @@ public class BuildAndDecodeTest {
         compareValuesFiles("values-notround/strings.xml");
     }
 
+    @Test
+    public void api26ConfigurationsTest() throws BrutException, IOException {
+        compareValuesFiles("values-widecg-v26/strings.xml");
+        compareValuesFiles("values-lowdr-v26/strings.xml");
+        compareValuesFiles("values-nowidecg-v26/strings.xml");
+        compareValuesFiles("values-vrheadset-v26/strings.xml");
+    }
+
+    @Test
+    public void fontTest() throws BrutException, IOException {
+        File fontXml = new File((sTestNewDir + "/res/font"), "lobster.xml");
+        File fontFile = new File((sTestNewDir + "/res/font"), "lobster_regular.otf");
+
+        // Per #1662, ensure font file is not encoded.
+        assertTrue(fontXml.isFile());
+        compareXmlFiles("/res/font/lobster.xml");
+
+        // If we properly skipped decoding the font (otf) file, this file should not exist
+        assertFalse((new File((sTestNewDir + "/res/values"), "fonts.xml")).isFile());
+        assertTrue(fontFile.isFile());
+    }
+
     @Test
     public void drawableNoDpiTest() throws BrutException, IOException {
         compareResFolder("drawable-nodpi");
@@ -343,6 +394,89 @@ public class BuildAndDecodeTest {
         assertEquals(controlImage.getRGB(30, 30), testImage.getRGB(30, 30));
     }
 
+    @Test
+    public void issue1508Test() throws BrutException, IOException {
+        char slash = File.separatorChar;
+        String location = slash + "res" + slash + "drawable-xhdpi" + slash;
+
+        File control = new File((sTestOrigDir + location), "btn_zoom_up_normal.9.png");
+        File test = new File((sTestNewDir + location), "btn_zoom_up_normal.9.png");
+
+        BufferedImage controlImage = ImageIO.read(control);
+        BufferedImage testImage = ImageIO.read(test);
+
+        // 0, 0 = clear
+        assertEquals(controlImage.getRGB(0, 0), testImage.getRGB(0, 0));
+
+        // 30, 0 = black line
+        assertEquals(controlImage.getRGB(0, 30), testImage.getRGB(0, 30));
+
+        // 30, 30 = greyish button
+        assertEquals(controlImage.getRGB(30, 30), testImage.getRGB(30, 30));
+    }
+
+    @Test
+    public void issue1511Test() throws BrutException, IOException {
+        char slash = File.separatorChar;
+        String location = slash + "res" + slash + "drawable-xxhdpi" + slash;
+
+        File control = new File((sTestOrigDir + location), "textfield_activated_holo_dark.9.png");
+        File test = new File((sTestNewDir + location), "textfield_activated_holo_dark.9.png");
+
+        BufferedImage controlImage = ImageIO.read(control);
+        BufferedImage testImage = ImageIO.read(test);
+
+        // Check entire image as we cannot mess this up
+        final int w = controlImage.getWidth(),
+                  h = controlImage.getHeight();
+
+        final int[] controlImageGrid = controlImage.getRGB(0, 0, w, h, null, 0, w);
+        final int[] testImageGrid = testImage.getRGB(0, 0, w, h, null, 0, w);
+
+        for (int i = 0; i < controlImageGrid.length; i++) {
+            assertEquals("Image lost Optical Bounds at i = " + i, controlImageGrid[i], testImageGrid[i]);
+        }
+    }
+
+    @Test
+    public void robust9patchTest() throws BrutException, IOException {
+        String[] ninePatches = {"ic_notification_overlay.9.png", "status_background.9.png",
+                "search_bg_transparent.9.png", "screenshot_panel.9.png", "recents_lower_gradient.9.png"};
+
+        char slash = File.separatorChar;
+        String location = slash + "res" + slash + "drawable-xxhdpi" + slash;
+
+        for (String ninePatch : ninePatches) {
+            File control = new File((sTestOrigDir + location), ninePatch);
+            File test = new File((sTestNewDir + location), ninePatch);
+
+            BufferedImage controlImage = ImageIO.read(control);
+            BufferedImage testImage = ImageIO.read(test);
+
+            int w = controlImage.getWidth(), h = controlImage.getHeight();
+
+            // Check the entire horizontal line
+            for (int i = 1; i < w; i++) {
+                if (isTransparent(controlImage.getRGB(i, 0))) {
+                    assertTrue(isTransparent(testImage.getRGB(i, 0)));
+                } else {
+                    assertEquals("Image lost npTc chunk on image " + ninePatch + " at (x, y) (" + i + "," + 0 + ")",
+                            controlImage.getRGB(i, 0), testImage.getRGB(i, 0));
+                }
+            }
+
+            // Check the entire vertical line
+            for (int i = 1; i < h; i++) {
+                if (isTransparent(controlImage.getRGB(0, i))) {
+                    assertTrue(isTransparent(testImage.getRGB(0, i)));
+                } else {
+                    assertEquals("Image lost npTc chunk on image " + ninePatch + " at (x, y) (" + 0 + "," + i + ")",
+                            controlImage.getRGB(0, i), testImage.getRGB(0, i));
+                }
+            }
+        }
+    }
+
     @Test
     public void drawableXxhdpiTest() throws BrutException, IOException {
         compareResFolder("drawable-xxhdpi");
@@ -373,6 +507,17 @@ public class BuildAndDecodeTest {
         compareUnknownFiles();
     }
 
+    @Test
+    public void fileAssetTest() throws BrutException, IOException {
+        compareAssetsFolder("txt");
+    }
+
+    @Test
+    public void unicodeAssetTest() throws BrutException, IOException {
+        assumeTrue(! OSDetection.isWindows());
+        compareAssetsFolder("unicode-txt");
+    }
+
     @Test
     public void multipleDexTest() throws BrutException, IOException {
         compareBinaryFolder("/smali_classes2", false);
@@ -410,7 +555,7 @@ public class BuildAndDecodeTest {
 
         String location = tmp + path;
 
-        FileDirectory fileDirectory = new FileDirectory(sTestOrigDir + location);
+        FileDirectory fileDirectory = new FileDirectory(sTestOrigDir, location);
 
         Set<String> files = fileDirectory.getFiles(true);
         for (String filename : files) {
@@ -434,6 +579,10 @@ public class BuildAndDecodeTest {
         compareBinaryFolder(File.separatorChar + path, false);
     }
 
+    private void compareAssetsFolder(String path) throws BrutException, IOException {
+        compareBinaryFolder(File.separatorChar + "assets" + File.separatorChar + path, false);
+    }
+
     private void compareValuesFiles(String path) throws BrutException {
         compareXmlFiles("res/" + path, new ElementNameAndAttributeQualifier("name"));
     }
@@ -448,6 +597,10 @@ public class BuildAndDecodeTest {
         assertTrue(f.isDirectory());
     }
 
+    private boolean isTransparent(int pixel) {
+        return pixel >> 24 == 0x00;
+    }
+
     private void compareXmlFiles(String path, ElementQualifier qualifier) throws BrutException {
         DetailedDiff diff;
         try {
